#include <Adafruit_NeoPixel.h>

//#include "piezo_buzzer.h"
#include "song_lights.h"
#include "optical_sensor_hackthon.h"


#define TICK_PIN 9
#define LED_PIN 13

// 4 - orange wire
// 5 - violet
// 3.3v - white
// gnd - yelow
// 2 - green

#define PB_PIN 3
#define SHOCK_ANALOG 0 // pin A0
#define SHOCK_BUFFER 100

#define MAX_DURATIONS 10


#define SECOND 1000 // ms in one second


#define DURATION_MAX_SCORE 100 // maximum score for accurate duration between knocks


#define TIMEOUT (3*SECOND)

int recorded[MAX_DURATIONS] = {0};
int mapped_recorded[MAX_DURATIONS] = {0};
int recorded_max_score = 0;

// In milliseconds
int user_durations[MAX_DURATIONS] = {0};

// Input from user
int led_state = LOW;

// shock sensor
int shock_max, shock_min;
bool knock_timeout = false;

void setup() {

   Serial.begin(9600);

   pinMode(TICK_PIN,OUTPUT);
   pinMode(LED_PIN,OUTPUT); 
   pinMode(PB_PIN, INPUT_PULLUP);

   setup_optic();

  // initial level
  recorded[0] = SECOND;
  recorded[1] = SECOND / 4;
  recorded[2] = SECOND / 4;
  recorded[3] = SECOND / 4;
  map_level(mapped_recorded, recorded);
  precalculate_max_scores();

  // put your setup code here, to run once:
  init_song_lights();

  // "Ground" for pixels, since we ran out of ground pins and are lazy
  pinMode(4, OUTPUT);
  digitalWrite(4,LOW);

  Serial.println("Playing intro song");

   // play intro sequence
   play_intro_song();
   delay(1 * SECOND);
}

void loop() 
{
  turnOffLights();

  Serial.println("Waiting for user to start game... ");

  if (get_user_input() == SWIPE_CHOICE) {
    Serial.println("User swiped");
    play_game();
    
  } else { /* HOLD_CHOISE */
    Serial.println("User hold");
    record_sequence();
  }
}





void play_game() 
{

  Serial.println("Starting game");

  // shock noise calibration
  calibrate_shock();

  int score = 0;

  // play sounds (curr_lvl)
  play_recorded();
    
  store_user_delays(false); // record mode is false
   
  // compare results and compute score and store score into array
  // calculate total score
  score = calculate_score();

  Serial.print("Score is: ");
  Serial.print(score);
  Serial.print("/");
  Serial.println(recorded_max_score);
  
  // display score here
  displayScore(score, recorded_max_score);
   
  // Displaying the score for 5 seconds
  delay(5 * SECOND);

  
}


void record_sequence() 
{
  Serial.println("Starting recording");


  // shock noise calibration
  calibrate_shock();

  Serial.println("Please record...");


  store_user_delays(true); // record mode is true

  // copy user to recorded
  for (int curr_duration = 0; curr_duration < MAX_DURATIONS; curr_duration++)
  {
    recorded[curr_duration] = user_durations[curr_duration];
  }

 Serial.println("Printing recorded data");
 Serial.print("Recorded: ");
 print_seq(recorded);
 Serial.print("Mapped recorded: ");
 print_seq(mapped_recorded);
 Serial.print("Max score: ");
 Serial.println(recorded_max_score);


  map_level(mapped_recorded, recorded);
  precalculate_max_scores();
  
}


void tick() {
    turnOnLights(strip.Color(150, 150, 150));
    tone(TICK_PIN, 1000, 20);
    delay(30);
    turnOffLights();
}

unsigned long detect_shock(unsigned int timeout_millis)
{
  unsigned long now = millis();

  while(true)
  {
    int analog_value = analogRead(SHOCK_ANALOG);
  
    if ((analog_value > shock_max + SHOCK_BUFFER) || (analog_value < shock_min - SHOCK_BUFFER))
    {
      break;
    }

    // if timeout, set and exit
    if ( (millis() - now) >= timeout_millis )
    {
      knock_timeout = true;
      break;
    }
  
  } // while

  return (millis() - now);
}


void calibrate_shock()
{
  Serial.println("Starting calibration");

  unsigned long start = millis();
  int sensorValue;

  shock_max = 0;
  shock_min = 1023;
  
  // calibrate during the first one seconds
  while (millis() - start < 1 * SECOND) {
    sensorValue = analogRead(SHOCK_ANALOG);

    // record the maximum sensor value
    if (sensorValue > shock_max) {
      shock_max = sensorValue;
      Serial.print("Shock MAX set: ");
      Serial.println(shock_max);
    }

    // record the minimum sensor value
    if (sensorValue < shock_min) {
      shock_min = sensorValue;
      Serial.print("Shock MIN set: ");
      Serial.println(shock_min);     
    }
  } //while

  Serial.println("Finished calibration");
}




void play_recorded() 
{
  for (int curr_duration = 0; curr_duration < MAX_DURATIONS; curr_duration++) {

    if( recorded[curr_duration] == 0) {
      break;
    }

    // play sound with buzzer
    Serial.println("Tick");
    tick();
    

    // wait 
    delay( ( recorded[curr_duration] ));
  }

  Serial.println("Tick");
  tick();

}


void store_user_delays(bool recording_mode)
{

   Serial.print("Store user delay. Recording mode is ");
   Serial.println(recording_mode);

  
  int knock_delay = 0;

  if (recording_mode == true) {
    colorWipe( strip.Color(255, 0, 0), 50);
    turnOnLights( strip.Color(255, 0, 0));
  } else {
    colorWipe( strip.Color(0, 255, 0), 50);
    turnOnLights( strip.Color(0, 255, 0));
  }
  
  for (int curr_duration = 0; curr_duration < MAX_DURATIONS; curr_duration++)
  {
    user_durations[curr_duration] = 0;
  }

  // First knock - no timeout
  //
  knock_timeout = false;
  knock_delay = detect_shock(TIMEOUT);
  Serial.println("Detected first knock");
  delay(100);

  // Read delays
  for (int curr_duration = 0; curr_duration < MAX_DURATIONS; curr_duration++) 
  {
    if (recording_mode == false && recorded[curr_duration] == 0) {
      break;
    }
    
    knock_timeout = false;
    knock_delay = detect_shock(TIMEOUT);
    if (knock_timeout)
    {
      break;
    } else 
    {
      Serial.print("Detected knock: ");
      Serial.println(knock_delay);

      // notice: we skip the duration between start and first tick
      user_durations[curr_duration] = knock_delay;
      delay(100);
    } // if knock_timeout
  } // for


  if (recording_mode == true) {
    colorWipe( strip.Color(255, 0, 0), 50);
    turnOffLights();

  } else {
    colorWipe( strip.Color(0, 255, 0), 50);
     turnOffLights();

  }  
}


int calculate_score()
{
   // use user_durations to calc score of current level
   //(LEVELS[curr_lvl][curr_duration]
   // mapped_levels
   
  int mapped_durations[MAX_DURATIONS] = {0};

  map_level(mapped_durations, user_durations);

  // TODO: do once in the record_sequence and in setup
  //map_level(mapped_recorded, recorded);
  //precalculate_max_scores();

  int penalty = 0;

  for (int curr_duration = 0; curr_duration < MAX_DURATIONS; curr_duration++) {

    Serial.print("penalty ");
    Serial.print(mapped_recorded[curr_duration]);
    Serial.print(" ");
    Serial.print(mapped_durations[curr_duration]);
    Serial.print(" ");
    Serial.print((abs(mapped_durations[curr_duration] - mapped_recorded[curr_duration]) * 100 ) / mapped_recorded[curr_duration]);
    Serial.println(" ");
//    Serial.print((abs(mapped_durations[curr_duration] - mapped_levels[level][curr_duration]) * 100 ));
//    Serial.print(" ");
//    Serial.println((abs(mapped_durations[curr_duration] - mapped_levels[level][curr_duration]) * 100 ) / mapped_levels[level][curr_duration]);
//    
    if (mapped_recorded[curr_duration] != 0 ) {
      penalty += abs(mapped_durations[curr_duration] - mapped_recorded[curr_duration]);
      //penalty += (abs(mapped_durations[curr_duration] - mapped_recorded[curr_duration]) * 100 ) / mapped_recorded[curr_duration];
    }
  }
  Serial.print("total penalty ");
  Serial.println(penalty);
  int calculated_score =  recorded_max_score - penalty;


  return calculated_score;
   
}



void precalculate_max_scores()
{
  recorded_max_score = 0;
    
  for (int curr_duration = 0; curr_duration < MAX_DURATIONS; curr_duration++) {
     if (recorded[curr_duration] == 0) {
        break;
      }
      
      recorded_max_score += DURATION_MAX_SCORE; 
  }
}

void map_level(int* to_mapped_level,const int* from_level) {
  int max_duration = from_level[0];
  for (int i = 1; i < MAX_DURATIONS; i++) {
    max_duration = max(max_duration, from_level[i]);
  }

  for (int i = 0; i < MAX_DURATIONS; i++) {    
    if (max_duration == 0) {
      to_mapped_level[i] = 0;
    } else {
      to_mapped_level[i] = map(from_level[i], 0, max_duration, 0, SHOCK_BUFFER);      
    }

  }
}


void print_seq(int* seq) {
 for (int i = 0; i < MAX_DURATIONS; i++) {    
    Serial.print(seq[i]);
    Serial.print(" ");
 }
 Serial.println("");
}

